@@ -34,9 +34,11 @@ mkdir -p ${DIR}
 ARCH=$( uname -m )
 build_system=${build_system:-autotools}
 if [ "${build_system}" = "cmake" ]; then
+    cmake --version
     mkdir build
     pushd build
-    cmake --log-level=VERBOSE ${CMAKE_OPTIONS:-} ..
+    cmake --log-level=VERBOSE --warn-uninitialized -Werror=dev \
+      ${CMAKE_OPTIONS:-} ..
     cmake --build .
     cpack
     popd
