@@ -1,5 +1,7 @@
-/* $Id$ $Revision$ */
-/* vim:set shiftwidth=4 ts=8: */
+/**
+ * @file
+ * @brief Network Simplex algorithm for ranking nodes of a DAG, @ref rank, @ref rank2
+ */
 
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
@@ -8,18 +10,12 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#include <common/render.h>
+#include <stdbool.h>
 
-/* 
- * Network Simplex Algorithm for Ranking Nodes of a DAG
- */
-
-#include "render.h"
-#include <setjmp.h>
-
-static int init_graph(graph_t *);
 static void dfs_cutval(node_t * v, edge_t * par);
 static int dfs_range(node_t * v, edge_t * par, int low);
 static int x_val(edge_t * e, node_t * v, int dir);
@@ -29,10 +25,9 @@ static void check_cycles(graph_t * g);
 
 #define LENGTH(e)		(ND_rank(aghead(e)) - ND_rank(agtail(e)))
 #define SLACK(e)		(LENGTH(e) - ED_minlen(e))
-#define SEQ(a,b,c)		(((a) <= (b)) && ((b) <= (c)))
+#define SEQ(a,b,c)		((a) <= (b) && (b) <= (c))
 #define TREE_EDGE(e)	(ED_tree_index(e) >= 0)
 
-static jmp_buf jbuf;
 static graph_t *G;
 static int N_nodes, N_edges;
 static int Minrank, Maxrank;
@@ -42,19 +37,19 @@ static int Search_size;
 static nlist_t Tree_node;
 static elist Tree_edge;
 
-static void add_tree_edge(edge_t * e)
+static int add_tree_edge(edge_t * e)
 {
     node_t *n;
     //fprintf(stderr,"add tree edge %p %s ", (void*)e, agnameof(agtail(e))) ; fprintf(stderr,"%s\n", agnameof(aghead(e))) ;
     if (TREE_EDGE(e)) {
 	agerr(AGERR, "add_tree_edge: missing tree edge\n");
-	longjmp (jbuf, 1);
+	return -1;
     }
     ED_tree_index(e) = Tree_edge.size;
     Tree_edge.list[Tree_edge.size++] = e;
-    if (ND_mark(agtail(e)) == FALSE)
+    if (!ND_mark(agtail(e)))
 	Tree_node.list[Tree_node.size++] = agtail(e);
-    if (ND_mark(aghead(e)) == FALSE)
+    if (!ND_mark(aghead(e)))
 	Tree_node.list[Tree_node.size++] = aghead(e);
     n = agtail(e);
     ND_mark(n) = TRUE;
@@ -62,7 +57,7 @@ static void add_tree_edge(edge_t * e)
     ND_tree_out(n).list[ND_tree_out(n).size] = NULL;
     if (ND_out(n).list[ND_tree_out(n).size - 1] == 0) {
 	agerr(AGERR, "add_tree_edge: empty outedge list\n");
-	longjmp (jbuf, 1);
+	return -1;
     }
     n = aghead(e);
     ND_mark(n) = TRUE;
@@ -70,8 +65,9 @@ static void add_tree_edge(edge_t * e)
     ND_tree_in(n).list[ND_tree_in(n).size] = NULL;
     if (ND_in(n).list[ND_tree_in(n).size - 1] == 0) {
 	agerr(AGERR, "add_tree_edge: empty inedge list\n");
-	longjmp (jbuf, 1);
+	return -1;
     }
+    return 0;
 }
 
 static void exchange_tree_edges(edge_t * e, edge_t * f)
@@ -84,14 +80,14 @@ static void exchange_tree_edges(edge_t * e, edge_t * f)
     ED_tree_index(e) = -1;
 
     n = agtail(e);
-    i = --(ND_tree_out(n).size);
+    i = --ND_tree_out(n).size;
     for (j = 0; j <= i; j++)
 	if (ND_tree_out(n).list[j] == e)
 	    break;
     ND_tree_out(n).list[j] = ND_tree_out(n).list[i];
     ND_tree_out(n).list[i] = NULL;
     n = aghead(e);
-    i = --(ND_tree_in(n).size);
+    i = --ND_tree_in(n).size;
     for (j = 0; j <= i; j++)
 	if (ND_tree_in(n).list[j] == e)
 	    break;
@@ -128,7 +124,7 @@ void init_rank(void)
 	for (i = 0; (e = ND_in(v).list[i]); i++)
 	    ND_rank(v) = MAX(ND_rank(v), ND_rank(agtail(e)) + ED_minlen(e));
 	for (i = 0; (e = ND_out(v).list[i]); i++) {
-	    if (--(ND_priority(aghead(e))) <= 0)
+	    if (--ND_priority(aghead(e)) <= 0)
 		enqueue(Q, aghead(e));
 	}
     }
@@ -186,10 +182,10 @@ static void dfs_enter_outedge(node_t * v)
     edge_t *e;
 
     for (i = 0; (e = ND_out(v).list[i]); i++) {
-	if (TREE_EDGE(e) == FALSE) {
+	if (!TREE_EDGE(e)) {
 	    if (!SEQ(Low, ND_lim(aghead(e)), Lim)) {
 		slack = SLACK(e);
-		if ((slack < Slack) || (Enter == NULL)) {
+		if (slack < Slack || Enter == NULL) {
 		    Enter = e;
 		    Slack = slack;
 		}
@@ -208,10 +204,10 @@ static void dfs_enter_inedge(node_t * v)
     edge_t *e;
 
     for (i = 0; (e = ND_in(v).list[i]); i++) {
-	if (TREE_EDGE(e) == FALSE) {
+	if (!TREE_EDGE(e)) {
 	    if (!SEQ(Low, ND_lim(agtail(e)), Lim)) {
 		slack = SLACK(e);
-		if ((slack < Slack) || (Enter == NULL)) {
+		if (slack < Slack || Enter == NULL) {
 		    Enter = e;
 		    Slack = slack;
 		}
@@ -219,7 +215,7 @@ static void dfs_enter_inedge(node_t * v)
 	} else if (ND_lim(agtail(e)) < ND_lim(v))
 	    dfs_enter_inedge(agtail(e));
     }
-    for (i = 0; (e = ND_tree_out(v).list[i]) && (Slack > 0); i++)
+    for (i = 0; (e = ND_tree_out(v).list[i]) && Slack > 0; i++)
 	if (ND_lim(aghead(e)) < ND_lim(v))
 	    dfs_enter_inedge(aghead(e));
 }
@@ -277,15 +273,19 @@ static int tight_subtree_search(Agnode_t *v, subtree_t *st)
     ND_subtree_set(v,st);
     for (i = 0; (e = ND_in(v).list[i]); i++) {
         if (TREE_EDGE(e)) continue;
-        if ((ND_subtree(agtail(e)) == 0) && (SLACK(e) == 0)) {
-               add_tree_edge(e);
+        if (ND_subtree(agtail(e)) == 0 && SLACK(e) == 0) {
+               if (add_tree_edge(e) != 0) {
+                   return -1;
+               }
                rv += tight_subtree_search(agtail(e),st);
         }
     }
     for (i = 0; (e = ND_out(v).list[i]); i++) {
         if (TREE_EDGE(e)) continue;
-        if ((ND_subtree(aghead(e)) == 0) && (SLACK(e) == 0)) {
-               add_tree_edge(e);
+        if (ND_subtree(aghead(e)) == 0 && SLACK(e) == 0) {
+               if (add_tree_edge(e) != 0) {
+                   return -1;
+               }
                rv += tight_subtree_search(aghead(e),st);
         }
     }
@@ -298,6 +298,10 @@ static subtree_t *find_tight_subtree(Agnode_t *v)
     rv = NEW(subtree_t);
     rv->rep = v;
     rv->size = tight_subtree_search(v,rv);
+    if (rv->size < 0) {
+        free(rv);
+        return NULL;
+    }
     rv->par = rv;
     return rv;
 }
@@ -310,7 +314,7 @@ typedef struct STheap_s {
 static subtree_t *STsetFind(Agnode_t *n0)
 {
   subtree_t *s0 = ND_subtree(n0);
-  while  (s0->par && (s0->par != s0)) {
+  while  (s0->par && s0->par != s0) {
     if (s0->par->par) {s0->par = s0->par->par;}  /* path compression for the code weary */
     s0 = s0->par;
   }
@@ -321,10 +325,10 @@ static subtree_t *STsetUnion(subtree_t *s0, subtree_t *s1)
 {
   subtree_t *r0, *r1, *r;
 
-  for (r0 = s0; r0->par && (r0->par != r0); r0 = r0->par);
-  for (r1 = s1; r1->par && (r1->par != r1); r1 = r1->par);
+  for (r0 = s0; r0->par && r0->par != r0; r0 = r0->par);
+  for (r1 = s1; r1->par && r1->par != r1; r1 = r1->par);
   if (r0 == r1) return r0;  /* safety code but shouldn't happen */
-  assert((r0->heap_index > -1) || (r1->heap_index > -1));
+  assert(r0->heap_index > -1 || r1->heap_index > -1);
   if (r1->heap_index == -1) r = r0;
   else if (r0->heap_index == -1) r = r1;
   else if (r1->size < r0->size) r = r0;
@@ -336,8 +340,6 @@ static subtree_t *STsetUnion(subtree_t *s0, subtree_t *s1)
   return r;
 }
 
-#define INCIDENT(e,treeset)  ((STsetFind(agtail(e),treeset)) != STsetFind(aghead(e),treeset))
-
 /* find tightest edge to another tree incident on the given tree */
 static Agedge_t *inter_tree_edge_search(Agnode_t *v, Agnode_t *from, Agedge_t *best)
 {
@@ -352,7 +354,7 @@ static Agedge_t *inter_tree_edge_search(Agnode_t *v, Agnode_t *from, Agedge_t *b
       }
       else {
         if (STsetFind(aghead(e)) != ts) {   // encountered candidate edge
-          if ((best == 0) || (SLACK(e) < SLACK(best))) best = e;
+          if (best == 0 || SLACK(e) < SLACK(best)) best = e;
         }
         /* else ignore non-tree edge between nodes in the same tree */
       }
@@ -365,7 +367,7 @@ static Agedge_t *inter_tree_edge_search(Agnode_t *v, Agnode_t *from, Agedge_t *b
       }
       else {
         if (STsetFind(agtail(e)) != ts) {
-          if ((best == 0) || (SLACK(e) < SLACK(best))) best = e;
+          if (best == 0 || SLACK(e) < SLACK(best)) best = e;
         }
       }
     }
@@ -375,7 +377,7 @@ static Agedge_t *inter_tree_edge_search(Agnode_t *v, Agnode_t *from, Agedge_t *b
 static Agedge_t *inter_tree_edge(subtree_t *tree)
 {
     Agedge_t *rv;
-    rv = inter_tree_edge_search(tree->rep, (Agnode_t *)0, (Agedge_t *)0);
+    rv = inter_tree_edge_search(tree->rep, NULL, NULL);
     return rv;
 }
 
@@ -390,9 +392,9 @@ void STheapify(STheap_t *heap, int i)
     do {
         left = 2*(i+1)-1;
         right = 2*(i+1);
-        if ((left < heap->size) && (elt[left]->size < elt[i]->size)) smallest = left;
+        if (left < heap->size && elt[left]->size < elt[i]->size) smallest = left;
         else smallest = i;
-        if ((right < heap->size) && (elt[right]->size < elt[smallest]->size)) smallest = right;
+        if (right < heap->size && elt[right]->size < elt[smallest]->size) smallest = right;
         else smallest = i;
         if (smallest != i) {
             subtree_t *temp;
@@ -469,13 +471,15 @@ subtree_t *merge_trees(Agedge_t *e)   /* entering tree edge */
 
   if (t0->heap_index == -1) {   // move t0
     delta = SLACK(e);
-    tree_adjust(t0->rep,(Agnode_t*)0,delta);
+    tree_adjust(t0->rep,NULL,delta);
   }
   else {  // move t1
     delta = -SLACK(e);
-    tree_adjust(t1->rep,0,delta);
+    tree_adjust(t1->rep,NULL,delta);
+  }
+  if (add_tree_edge(e) != 0) {
+    return NULL;
   }
-  add_tree_edge(e);
   rv = STsetUnion(t0,t1);
   
   return rv;
@@ -493,7 +497,7 @@ int feasible_tree(void)
   Agedge_t *ee;
   subtree_t **tree, *tree0, *tree1;
   int i, subtree_count = 0;
-  STheap_t *heap;
+  STheap_t *heap = NULL;
   int error = 0;
 
   /* initialization */
@@ -506,6 +510,10 @@ int feasible_tree(void)
   for (n = GD_nlist(G); n; n = ND_next(n)) {
         if (ND_subtree(n) == 0) {
                 tree[subtree_count] = find_tight_subtree(n);
+                if (tree[subtree_count] == NULL) {
+                    error = 2;
+                    goto end;
+                }
                 subtree_count++;
         }
   }
@@ -519,24 +527,23 @@ int feasible_tree(void)
       break;
     }
     tree1 = merge_trees(ee);
+    if (tree1 == NULL) {
+      error = 2;
+      break;
+    }
     STheapify(heap,tree1->heap_index);
   }
 
+end:
   free(heap);
   for (i = 0; i < subtree_count; i++) free(tree[i]);
   free(tree);
-  if (error) return 1;
+  if (error) return error;
   assert(Tree_edge.size == N_nodes - 1);
   init_cutvalues();
   return 0;
 }
 
-/* utility functions for debugging */
-static subtree_t *nd_subtree(Agnode_t *n) {return ND_subtree(n);}
-static int nd_priority(Agnode_t *n) {return ND_priority(n);}
-static int nd_rank(Agnode_t *n) {return ND_rank(n);}
-static int ed_minlen(Agedge_t *e) {return ED_minlen(e);}
-
 /* walk up from v to LCA(v,w), setting new cutvalues. */
 static Agnode_t *treeupdate(Agnode_t * v, Agnode_t * w, int cutvalue, int dir)
 {
@@ -548,7 +555,7 @@ static Agnode_t *treeupdate(Agnode_t * v, Agnode_t * w, int cutvalue, int dir)
 	if (v == agtail(e))
 	    d = dir;
 	else
-	    d = NOT(dir);
+	    d = !dir;
 	if (d)
 	    ED_cutvalue(e) += cutvalue;
 	else
@@ -578,7 +585,7 @@ static void rerank(Agnode_t * v, int delta)
 /* e is the tree edge that is leaving and f is the nontree edge that
  * is entering.  compute new cut values, ranks, and exchange e and f.
  */
-static void 
+static int
 update(edge_t * e, edge_t * f)
 {
     int cutvalue, delta;
@@ -608,12 +615,13 @@ update(edge_t * e, edge_t * f)
     lca = treeupdate(agtail(f), aghead(f), cutvalue, 1);
     if (treeupdate(aghead(f), agtail(f), cutvalue, 0) != lca) {
 	agerr(AGERR, "update: mismatched lca in treeupdates\n");
-	longjmp (jbuf, 1);
+	return 2;
     }
     ED_cutvalue(f) = -cutvalue;
     ED_cutvalue(e) = 0;
     exchange_tree_edges(e, f);
     dfs_range(lca, ND_par(lca), ND_low(lca));
+    return 0;
 }
 
 static void scan_and_normalize(void)
@@ -640,7 +648,7 @@ static void
 freeTreeList (graph_t* g)
 {
     node_t *n;
-    for (n = GD_nlist(G); n; n = ND_next(n)) {
+    for (n = GD_nlist(g); n; n = ND_next(n)) {
 	free_list(ND_tree_in(n));
 	free_list(ND_tree_out(n));
 	ND_mark(n) = FALSE;
@@ -671,11 +679,23 @@ static void LR_balance(void)
 }
 
 static int decreasingrankcmpf(node_t **n0, node_t **n1) {
-  return ND_rank(*n1) - ND_rank(*n0);
+  if (ND_rank(*n1) < ND_rank(*n0)) {
+    return -1;
+  }
+  if (ND_rank(*n1) > ND_rank(*n0)) {
+    return 1;
+  }
+  return 0;
 }
 
 static int increasingrankcmpf(node_t **n0, node_t **n1) {
-  return ND_rank(*n0) - ND_rank(*n1);
+  if (ND_rank(*n0) < ND_rank(*n1)) {
+    return -1;
+  }
+  if (ND_rank(*n0) > ND_rank(*n1)) {
+    return 1;
+  }
+  return 0;
 }
 
 static void TB_balance(void)
@@ -697,16 +717,22 @@ static void TB_balance(void)
          if (streq(s,"min")) adj = 1;
          else if (streq(s,"max")) adj = 2;
          if (adj) for (n = GD_nlist(G); n; n = ND_next(n))
-              if (ND_node_type(n) == NORMAL)
-                if (ND_out(n).size == 0)
-                   ND_rank(n) = ((adj == 1)? Minrank : Maxrank);
+              if (ND_node_type(n) == NORMAL) {
+                if (ND_in(n).size == 0 && adj == 1) {
+                   ND_rank(n) = Minrank;
+                }
+                if (ND_out(n).size == 0 && adj == 2) {
+                   ND_rank(n) = Maxrank;
+                }
+              }
     }
     for (ii = 0, n = GD_nlist(G); n; ii++, n = ND_next(n)) {
       Tree_node.list[ii] = n;
     }
     Tree_node.size = ii;
     qsort(Tree_node.list, Tree_node.size, sizeof(Tree_node.list[0]),
-        adj > 1? decreasingrankcmpf : increasingrankcmpf);
+        adj > 1? (int(*)(const void*,const void*))decreasingrankcmpf
+               : (int(*)(const void*,const void*))increasingrankcmpf);
     for (i = 0; i < Tree_node.size; i++) {
         n = Tree_node.list[i];
         if (ND_node_type(n) == NORMAL)
@@ -779,7 +805,7 @@ static int init_graph(graph_t * g)
 	    ED_cutvalue(e) = 0;
 	    ED_tree_index(e) = -1;
 	    if (feasible
-		&& (ND_rank(aghead(e)) - ND_rank(agtail(e)) < ED_minlen(e)))
+		&& ND_rank(aghead(e)) - ND_rank(agtail(e)) < ED_minlen(e))
 		feasible = FALSE;
 	}
 	ND_tree_in(n).list = N_NEW(i + 1, edge_t *);
@@ -843,29 +869,34 @@ int rank2(graph_t * g, int balance, int maxiter, int search_size)
     feasible = init_graph(g);
     if (!feasible)
 	init_rank();
-    if (maxiter <= 0) {
-	freeTreeList (g);
-	return 0;
-    }
 
     if (search_size >= 0)
 	Search_size = search_size;
     else
 	Search_size = SEARCHSIZE;
 
-    if (setjmp (jbuf)) {
-	return 2;
+    {
+	int err = feasible_tree();
+	if (err != 0) {
+	    freeTreeList (g);
+	    return err;
+	}
+    }
+    if (maxiter <= 0) {
+	freeTreeList (g);
+	return 0;
     }
 
-    if (feasible_tree()) {
-	freeTreeList (g);
-	return 1;
-    }
     while ((e = leave_edge())) {
+	int err;
 	f = enter_edge(e);
-	update(e, f);
+	err = update(e, f);
+	if (err != 0) {
+	    freeTreeList (g);
+	    return err;
+	}
 	iter++;
-	if (Verbose && (iter % 100 == 0)) {
+	if (Verbose && iter % 100 == 0) {
 	    if (iter % 1000 == 100)
 		fputs(ns, stderr);
 	    fprintf(stderr, "%d ", iter);
@@ -1021,7 +1052,7 @@ void tchk(void)
 		fprintf(stderr, "not a tight tree %p", e);
 	}
     }
-    if ((n_cnt != Tree_node.size) || (e_cnt != Tree_edge.size))
+    if (n_cnt != Tree_node.size || e_cnt != Tree_edge.size)
 	fprintf(stderr, "something missing\n");
 }
 
@@ -1091,7 +1122,7 @@ static char* dump_node (node_t* n)
     static char buf[50];
 
     if (ND_node_type(n)) {
-	sprintf(buf, "%p", n);
+	snprintf(buf, sizeof(buf), "%p", n);
 	return buf;
     }
     else
@@ -1129,7 +1160,7 @@ static node_t *checkdfs(graph_t* g, node_t * n)
     if (ND_mark(n))
 	return 0;
     ND_mark(n) = TRUE;
-    ND_onstack(n) = TRUE;
+    ND_onstack(n) = true;
     for (i = 0; (e = ND_out(n).list[i]); i++) {
 	w = aghead(e);
 	if (ND_onstack(w)) {
@@ -1141,7 +1172,7 @@ static node_t *checkdfs(graph_t* g, node_t * n)
 	    return w;
 	}
 	else {
-	    if (ND_mark(w) == FALSE) {
+	    if (!ND_mark(w)) {
 		x = checkdfs(g, w);
 		if (x) {
 		    fprintf(stderr,"unwind %lx %s(%lx)\n",
@@ -1156,15 +1187,17 @@ static node_t *checkdfs(graph_t* g, node_t * n)
 	    }
 	}
     }
-    ND_onstack(n) = FALSE;
+    ND_onstack(n) = false;
     return 0;
 }
 
 void check_cycles(graph_t * g)
 {
     node_t *n;
-    for (n = GD_nlist(g); n; n = ND_next(n))
-	ND_mark(n) = ND_onstack(n) = FALSE;
+    for (n = GD_nlist(g); n; n = ND_next(n)) {
+	ND_mark(n) = FALSE;
+	ND_onstack(n) = false;
+    }
     for (n = GD_nlist(g); n; n = ND_next(n))
 	checkdfs(g, n);
 }
