@@ -1,6 +1,3 @@
-/* $Id$ $Revision$ */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,47 +5,38 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include "config.h"
 
+#include	<stdbool.h>
+#include	<stddef.h>
 #include	<string.h>
-#include        <sys/types.h>
-#include        <sys/stat.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#else
-#include <compat_unistd.h>
-#endif
+#include	<unistd.h>
 
 #ifdef ENABLE_LTDL
 #include	<ltdl.h>
 #endif
 
-#include	<agxbuf.h>
-#include        "memory.h"
-#include        "types.h"
-#include        "gvplugin.h"
-#include        "gvcjob.h"
-#include        "gvcint.h"
-#include        "gvcproc.h"
-#include        "gvio.h"
+#include	<cgraph/agxbuf.h>
+#include        <common/memory.h>
+#include        <common/types.h>
+#include        <gvc/gvplugin.h>
+#include        <gvc/gvcjob.h>
+#include        <gvc/gvcint.h>
+#include        <gvc/gvcproc.h>
+#include        <gvc/gvio.h>
 
-#include	"const.h"
-
-#ifndef HAVE_STRCASECMP
-extern int strcasecmp(const char *s1, const char *s2);
-#endif
-
-#ifdef _WIN32
-#define strdup(x) _strdup(x)
-#endif
+#include	<common/const.h>
+#include <cgraph/alloc.h>
+#include <cgraph/strcasecmp.h>
+#include <cgraph/strview.h>
 
 /*
  * Define an apis array of name strings using an enumerated api_t as index.
  * The enumerated type is defined gvplugin.h.  The apis array is
- * inititialized here by redefining ELEM and reinvoking APIS.
+ * initialized here by redefining ELEM and reinvoking APIS.
  */
 #define ELEM(x) #x,
 static char *api_names[] = { APIS };    /* "render", "layout", ... */
@@ -56,11 +44,9 @@ static char *api_names[] = { APIS };    /* "render", "layout", ... */
 #undef ELEM
 
 /* translate a string api name to its type, or -1 on error */
-api_t gvplugin_api(char *str)
+api_t gvplugin_api(const char *str)
 {
-    int api;
-
-    for (api = 0; api < ARRAY_SIZE(api_names); api++) {
+    for (size_t api = 0; api < ARRAY_SIZE(api_names); api++) {
         if (strcmp(str, api_names[api]) == 0)
             return (api_t) api;
     }
@@ -80,51 +66,57 @@ char *gvplugin_api_name(api_t api)
  * quality sorted within the type, then, if qualities are the same,
  * last install wins.
  */
-boolean gvplugin_install(GVC_t * gvc, api_t api, const char *typestr,
-                         int quality, gvplugin_package_t * package, gvplugin_installed_t * typeptr)
+bool gvplugin_install(GVC_t *gvc, api_t api, const char *typestr, int quality,
+                      gvplugin_package_t *package,
+                      gvplugin_installed_t *typeptr)
 {
     gvplugin_available_t *plugin, **pnext;
-#define TYPSIZ 63
-    char *p, pins[TYPSIZ + 1], pnxt[TYPSIZ + 1];
+    char *t;
 
-    strncpy(pins, typestr, TYPSIZ);
-    if ((p = strchr(pins, ':')))
-        *p = '\0';
+    /* duplicate typestr to later save in the plugin list */
+    t = strdup(typestr);
+    if (t == NULL)
+        return false;
+
+    // find the current plugin
+    const strview_t type = strview(typestr, ':');
 
     /* point to the beginning of the linked list of plugins for this api */
-    pnext = &(gvc->apis[api]);
+    pnext = &gvc->apis[api];
 
     /* keep alpha-sorted and insert new duplicates ahead of old */
     while (*pnext) {
-        strncpy(pnxt, (*pnext)->typestr, TYPSIZ);
-        if ((p = strchr(pnxt, ':')))
-            *p = '\0';
-        if (strcmp(pins, pnxt) <= 0)
+
+        // find the next plugin
+        const strview_t next_type = strview((*pnext)->typestr, ':');
+
+        if (strview_cmp(type, next_type) <= 0)
             break;
-        pnext = &((*pnext)->next);
+        pnext = &(*pnext)->next;
     }
 
     /* keep quality sorted within type and insert new duplicates ahead of old */
     while (*pnext) {
-        strncpy(pnxt, (*pnext)->typestr, TYPSIZ);
-        if ((p = strchr(pnxt, ':')))
-            *p = '\0';
-        if (strcmp(pins, pnxt) != 0)
+
+        // find the next plugin
+        const strview_t next_type = strview((*pnext)->typestr, ':');
+
+        if (!strview_eq(type, next_type))
             break;
         if (quality >= (*pnext)->quality)
             break;
-        pnext = &((*pnext)->next);
+        pnext = &(*pnext)->next;
     }
 
     plugin = GNEW(gvplugin_available_t);
     plugin->next = *pnext;
     *pnext = plugin;
-    plugin->typestr = typestr;
+    plugin->typestr = t;
     plugin->quality = quality;
     plugin->package = package;
     plugin->typeptr = typeptr;  /* null if not loaded */
 
-    return TRUE;
+    return true;
 }
 
 /* Activate a plugin description in the list of available plugins.
@@ -134,25 +126,25 @@ boolean gvplugin_install(GVC_t * gvc, api_t api, const char *typestr,
  * NB the quality value is not replaced as it might have been
  * manually changed in the config file.
  */
-static boolean gvplugin_activate(GVC_t * gvc, api_t api,
-                                 const char *typestr, char *name, char *path, gvplugin_installed_t * typeptr)
+static void gvplugin_activate(GVC_t * gvc, api_t api, const char *typestr,
+                              const char *name, const char *plugin_path,
+                              gvplugin_installed_t * typeptr)
 {
-    gvplugin_available_t **pnext;
+    gvplugin_available_t *pnext;
 
     /* point to the beginning of the linked list of plugins for this api */
-    pnext = &(gvc->apis[api]);
+    pnext = gvc->apis[api];
 
-    while (*pnext) {
-        if ((strcasecmp(typestr, (*pnext)->typestr) == 0)
-            && (strcasecmp(name, (*pnext)->package->name) == 0)
-            && ((*pnext)->package->path != 0)
-            && (strcasecmp(path, (*pnext)->package->path) == 0)) {
-            (*pnext)->typeptr = typeptr;
-            return TRUE;
+    while (pnext) {
+        if (strcasecmp(typestr, pnext->typestr) == 0
+            && strcasecmp(name, pnext->package->name) == 0
+            && pnext->package->path != 0
+            && strcasecmp(plugin_path, pnext->package->path) == 0) {
+            pnext->typeptr = typeptr;
+            return;
         }
-        pnext = &((*pnext)->next);
+        pnext = pnext->next;
     }
-    return FALSE;
 }
 
 gvplugin_library_t *gvplugin_library_load(GVC_t * gvc, char *path)
@@ -161,12 +153,11 @@ gvplugin_library_t *gvplugin_library_load(GVC_t * gvc, char *path)
     lt_dlhandle hndl;
     lt_ptr ptr;
     char *s, *sym;
-    int len;
+    size_t len;
     static char *p;
-    static int lenp;
+    static size_t lenp;
     char *libdir;
     char *suffix = "_LTX_library";
-    struct stat sb;
 
     if (!gvc->common.demand_loading)
         return NULL;
@@ -175,10 +166,7 @@ gvplugin_library_t *gvplugin_library_load(GVC_t * gvc, char *path)
     len = strlen(libdir) + 1 + strlen(path) + 1;
     if (len > lenp) {
         lenp = len + 20;
-        if (p)
-            p = grealloc(p, lenp);
-        else
-            p = gmalloc(lenp);
+        p = grealloc(p, lenp);
     }
 #ifdef _WIN32
     if (path[1] == ':') {
@@ -198,11 +186,11 @@ gvplugin_library_t *gvplugin_library_load(GVC_t * gvc, char *path)
     }
     hndl = lt_dlopen(p);
     if (!hndl) {
-        if ((stat(p, &sb)) == 0) {
+        if (access(p, R_OK) == 0) {
             agerr(AGWARN, "Could not load \"%s\" - %s\n", p, "It was found, so perhaps one of its dependents was not.  Try ldd.");
         }
         else {
-            agerr(AGWARN, "Could not load \"%s\" - %s\n", p, (char *) lt_dlerror());
+            agerr(AGWARN, "Could not load \"%s\" - %s\n", p, lt_dlerror());
         }
         return NULL;
     }
@@ -239,7 +227,7 @@ gvplugin_library_t *gvplugin_library_load(GVC_t * gvc, char *path)
         return NULL;
     }
     free(sym);
-    return (gvplugin_library_t *) (ptr);
+    return (gvplugin_library_t *)ptr;
 #else
     agerr(AGERR, "dynamic loading not available\n");
     return NULL;
@@ -261,13 +249,10 @@ gvplugin_library_t *gvplugin_library_load(GVC_t * gvc, char *path)
 */
 gvplugin_available_t *gvplugin_load(GVC_t * gvc, api_t api, const char *str)
 {
-    gvplugin_available_t **pnext, *rv;
+    gvplugin_available_t *pnext, *rv;
     gvplugin_library_t *library;
     gvplugin_api_t *apis;
     gvplugin_installed_t *types;
-#define TYPBUFSIZ 64
-    char reqtyp[TYPBUFSIZ], typ[TYPBUFSIZ];
-    char *reqdep, *dep = NULL, *reqpkg;
     int i;
     api_t apidep;
 
@@ -277,36 +262,45 @@ gvplugin_available_t *gvplugin_load(GVC_t * gvc, api_t api, const char *str)
     else
         apidep = api;
 
-    strncpy(reqtyp, str, TYPBUFSIZ - 1);
-    reqdep = strchr(reqtyp, ':');
-    if (reqdep) {
-        *reqdep++ = '\0';
-        reqpkg = strchr(reqdep, ':');
-        if (reqpkg)
-            *reqpkg++ = '\0';
-    } else
-        reqpkg = NULL;
+    const strview_t reqtyp = strview(str, ':');
+
+    strview_t reqdep = {0};
+
+    strview_t reqpkg = {0};
+
+    if (reqtyp.data[reqtyp.size] == ':') {
+        reqdep = strview(reqtyp.data + reqtyp.size + strlen(":"), ':');
+        if (reqdep.data[reqdep.size] == ':') {
+            reqpkg = strview(reqdep.data + reqdep.size + strlen(":"), '\0');
+        }
+    }
 
     /* iterate the linked list of plugins for this api */
-    for (pnext = &(gvc->apis[api]); *pnext; pnext = &((*pnext)->next)) {
-        strncpy(typ, (*pnext)->typestr, TYPBUFSIZ - 1);
-        dep = strchr(typ, ':');
-        if (dep)
-            *dep++ = '\0';
-        if (strcmp(typ, reqtyp))
-            continue;           /* types empty or mismatched */
-        if (dep && reqdep && strcmp(dep, reqdep))
-            continue;           /* dependencies not empty, but mismatched */
-        if (!reqpkg || strcmp(reqpkg, (*pnext)->package->name) == 0) {
-            /* found with no packagename constraints, or with required matching packagname */
+    for (pnext = gvc->apis[api]; pnext; pnext = pnext->next) {
+        const strview_t typ = strview(pnext->typestr, ':');
 
-            if (dep && (apidep != api)) /* load dependency if needed, continue if can't find */
-                if (!(gvplugin_load(gvc, apidep, dep)))
+        strview_t dep = {0};
+        if (typ.data[typ.size] == ':') {
+            dep = strview(typ.data + typ.size + strlen(":"), '\0');
+        }
+
+        if (!strview_eq(typ, reqtyp))
+            continue;           /* types empty or mismatched */
+        if (dep.data && reqdep.data) {
+            if (!strview_eq(dep, reqdep)) {
+                continue;           /* dependencies not empty, but mismatched */
+            }
+        }
+        if (!reqpkg.data || strview_str_eq(reqpkg, pnext->package->name)) {
+            // found with no packagename constraints, or with required matching packagename
+
+            if (dep.data && apidep != api) // load dependency if needed, continue if can't find
+                if (!gvplugin_load(gvc, apidep, dep.data))
                     continue;
             break;
         }
     }
-    rv = *pnext;
+    rv = pnext;
 
     if (rv && rv->typeptr == NULL) {
         library = gvplugin_library_load(gvc, rv->package->path);
@@ -341,70 +335,46 @@ gvplugin_available_t *gvplugin_load(GVC_t * gvc, api_t api, const char *str)
  */
 char *gvplugin_list(GVC_t * gvc, api_t api, const char *str)
 {
-    static int first = 1;
-    gvplugin_available_t **pnext, **plugin;
+    const gvplugin_available_t *pnext, *plugin;
     char *bp;
-    char *s, *p, *q, *typestr_last;
-    boolean new = TRUE;
+    bool new = true;
     static agxbuf xb;
 
     /* check for valid str */
     if (!str)
         return NULL;
 
-    if (first) {
-        agxbinit(&xb, 0, 0);
-        first = 0;
-    }
-
     /* does str have a :path modifier? */
-    s = strdup(str);
-    p = strchr(s, ':');
-    if (p)
-        *p++ = '\0';
+    const strview_t strv = strview(str, ':');
 
     /* point to the beginning of the linked list of plugins for this api */
-    plugin = &(gvc->apis[api]);
+    plugin = gvc->apis[api];
 
-    if (p) {                    /* if str contains a ':', and if we find a match for the type,
-                                   then just list the alternative paths for the plugin */
-        for (pnext = plugin; *pnext; pnext = &((*pnext)->next)) {
-            q = strdup((*pnext)->typestr);
-            if ((p = strchr(q, ':')))
-                *p++ = '\0';
+    if (strv.data[strv.size] == ':') { /* if str contains a ':', and if we find a match for the type,
+                                          then just list the alternative paths for the plugin */
+        for (pnext = plugin; pnext; pnext = pnext->next) {
+            const strview_t type = strview(pnext->typestr, ':');
             /* list only the matching type, or all types if s is an empty string */
-            if (!s[0] || strcasecmp(s, q) == 0) {
+            if (!str[0] || strview_case_eq(strv, type)) {
                 /* list each member of the matching type as "type:path" */
-                agxbputc(&xb, ' ');
-                agxbput(&xb, (*pnext)->typestr);
-                agxbputc(&xb, ':');
-                agxbput(&xb, (*pnext)->package->name);
-                new = FALSE;
+                agxbprint(&xb, " %s:%s", pnext->typestr, pnext->package->name);
+                new = false;
             }
-            free(q);
         }
     }
-    free(s);
     if (new) {                  /* if the type was not found, or if str without ':',
                                    then just list available types */
-        typestr_last = NULL;
-        for (pnext = plugin; *pnext; pnext = &((*pnext)->next)) {
+        strview_t type_last = {0};
+        for (pnext = plugin; pnext; pnext = pnext->next) {
             /* list only one instance of type */
-            q = strdup((*pnext)->typestr);
-            if ((p = strchr(q, ':')))
-                *p++ = '\0';
-            if (!typestr_last || strcasecmp(typestr_last, q) != 0) {
+            const strview_t type = strview(pnext->typestr, ':');
+            if (!type_last.data || !strview_case_eq(type_last, type)) {
                 /* list it as "type"  i.e. w/o ":path" */
-                agxbputc(&xb, ' ');
-                agxbput(&xb, q);
-                new = FALSE;
+                agxbprint(&xb, " %.*s", (int)type.size, type.data);
+                new = false;
             }
-            if (!typestr_last)
-                free(typestr_last);
-            typestr_last = q;
+            type_last = type;
         }
-        if (!typestr_last)
-            free(typestr_last);
     }
     if (new)
         bp = "";
@@ -427,10 +397,11 @@ char *gvplugin_list(GVC_t * gvc, api_t api, const char *str)
 char **gvPluginList(GVC_t * gvc, const char *kind, int *sz, const char *str)
 {
     int api;
-    gvplugin_available_t **pnext, **plugin;
+    const gvplugin_available_t *pnext, *plugin;
     int cnt = 0;
     char **list = NULL;
-    char *p, *q, *typestr_last;
+
+    (void)str;
 
     if (!kind)
         return NULL;
@@ -444,16 +415,14 @@ char **gvPluginList(GVC_t * gvc, const char *kind, int *sz, const char *str)
     }
 
     /* point to the beginning of the linked list of plugins for this api */
-    plugin = &(gvc->apis[api]);
-    typestr_last = NULL;
-    for (pnext = plugin; *pnext; pnext = &((*pnext)->next)) {
+    plugin = gvc->apis[api];
+    strview_t typestr_last = {0};
+    for (pnext = plugin; pnext; pnext = pnext->next) {
         /* list only one instance of type */
-        q = strdup((*pnext)->typestr);
-        if ((p = strchr(q, ':')))
-            *p++ = '\0';
-        if (!typestr_last || strcasecmp(typestr_last, q) != 0) {
+        strview_t q = strview(pnext->typestr, ':');
+        if (!typestr_last.data || !strview_case_eq(typestr_last, q)) {
             list = RALLOC(cnt + 1, list, char *);
-            list[cnt++] = q;
+            list[cnt++] = strview_str(q);
         }
         typestr_last = q;
     }
@@ -494,11 +463,11 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
     Agedge_t *e;
     Agsym_t *a;
     gvplugin_package_t *package;
-    gvplugin_available_t **pnext;
+    const gvplugin_available_t *pnext;
     char bufa[100], *buf1, *buf2, bufb[100], *p, *q, *lq, *t;
     int api, neededge_loadimage, neededge_device;
 
-    g = agopen("G", Agdirected, NIL(Agdisc_t *));
+    g = agopen("G", Agdirected, NULL);
     agattr(g, AGRAPH, "label", "");
     agattr(g, AGRAPH, "rankdir", "");
     agattr(g, AGRAPH, "rank", "");
@@ -536,9 +505,9 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
             agxset(ssg, a, "same");
             strcat(buf1, "_");
             buf2 = bufa + strlen(bufa);
-            for (pnext = &(gvc->apis[api]); *pnext; pnext = &((*pnext)->next)) {
-                if ((*pnext)->package == package) {
-                    t = q = strdup((*pnext)->typestr);
+            for (pnext = gvc->apis[api]; pnext; pnext = pnext->next) {
+                if (pnext->package == package) {
+                    t = q = gv_strdup(pnext->typestr);
                     if ((p = strchr(q, ':')))
                         *p++ = '\0';
                     /* Now p = renderer, e.g. "gd"
@@ -712,9 +681,9 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
             strcpy(buf1, api_names[api]);
             strcat(buf1, "_");
             buf2 = bufa + strlen(bufa);
-            for (pnext = &(gvc->apis[api]); *pnext; pnext = &((*pnext)->next)) {
-                if ((*pnext)->package == package) {
-                    t = q = strdup((*pnext)->typestr);
+            for (pnext = gvc->apis[api]; pnext; pnext = pnext->next) {
+                if (pnext->package == package) {
+                    t = q = gv_strdup(pnext->typestr);
                     if ((p = strchr(q, ':')))
                         *p++ = '\0';
                     /* Now p = renderer, e.g. "gd"
