@@ -38,7 +38,13 @@ static int cmpitem(Dt_t * d, int *p1, int *p2, Dtdisc_t * disc)
     (void)d;
     (void)disc;
 
-    return (*p1 - *p2);
+    if (*p1 < *p2) {
+        return -1;
+    }
+    if (*p1 > *p2) {
+        return 1;
+    }
+    return 0;
 }
 
 static Dtdisc_t constr = {
@@ -131,10 +137,10 @@ static void mapGraphs(graph_t * g, graph_t * cg, distfn dist)
     int delta;
 
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	tp = (nitem *) ND_alg(n);
+	tp = ND_alg(n);
 	t = tp->cnode;
 	for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
-	    hp = (nitem *) ND_alg(aghead(e));
+	    hp = ND_alg(aghead(e));
 	    delta = dist(&tp->bb, &hp->bb);
 	    h = hp->cnode;
 	    ce = agedge(cg, t, h, NULL, 1);
@@ -264,9 +270,7 @@ static graph_t *mkNConstraintG(graph_t * g, Dt_t * list,
 }
 /* mkConstraintG:
  */
-static graph_t *mkConstraintG(graph_t * g, Dt_t * list,
-			      intersectfn intersect, distfn dist)
-{
+static graph_t *mkConstraintG(Dt_t * list, intersectfn intersect, distfn dist) {
     nitem *p;
     nitem *nxt = NULL;
     nitem *nxp;
@@ -396,7 +400,7 @@ static void constrainX(graph_t* g, nitem* nlist, int nnodes, intersectfn ifn,
 	p++;
     }
     if (ortho)
-	cg = mkConstraintG(g, list, ifn, distX);
+	cg = mkConstraintG(list, ifn, distX);
     else
 	cg = mkNConstraintG(g, list, ifn, distX);
     rank(cg, 2, INT_MAX);
@@ -434,7 +438,7 @@ static void constrainY(graph_t* g, nitem* nlist, int nnodes, intersectfn ifn,
 	p++;
     }
     if (ortho)
-	cg = mkConstraintG(g, list, ifn, distY);
+	cg = mkConstraintG(list, ifn, distY);
     else
 	cg = mkNConstraintG(g, list, ifn, distY);
     rank(cg, 2, INT_MAX);
