@@ -1,3 +1,4 @@
+#include <algorithm>
 #include <cassert>
 #include <cmath>
 #include <limits>
@@ -347,21 +348,43 @@ SVG::SVGRect SVG::SVGElement::outline_bbox(bool throw_if_bbox_not_defined) {
       throw std::runtime_error{"Too few points for 'polyline' element"};
     }
 
-    // handle first and last point which may not be part of a corner
-    const SVG::SVGRect first_point_bbox = {
-        points.front().x - attributes.stroke_width / 2,
-        points.front().y - attributes.stroke_width / 2,
-        attributes.stroke_width,
-        attributes.stroke_width,
-    };
-    m_bbox->extend(first_point_bbox);
-    const SVG::SVGRect last_point_bbox = {
-        points.back().x - attributes.stroke_width / 2,
-        points.back().y - attributes.stroke_width / 2,
-        attributes.stroke_width,
-        attributes.stroke_width,
-    };
-    m_bbox->extend(last_point_bbox);
+    // handle first point which may not be part of a corner
+    {
+      const auto first_point = points.front();
+      const auto next_point = points[1];
+      const auto dx = first_point.x - next_point.x;
+      const auto dy = first_point.y - next_point.y;
+      const auto hypot = std::hypot(dx, dy);
+      const auto cosAlpha = dx / hypot;
+      const auto sinAlpha = dy / hypot;
+      // a polyline extends with half the stroke width in both perpendicular
+      // directions, but not along the line at its endpoints
+      const auto x_extension = std::abs(attributes.stroke_width / 2 * sinAlpha);
+      const auto y_extension = std::abs(attributes.stroke_width / 2 * cosAlpha);
+      const SVG::SVGRect first_point_bbox = {points.front().x - x_extension,
+                                             points.front().y - y_extension,
+                                             x_extension * 2, y_extension * 2};
+      m_bbox->extend(first_point_bbox);
+    }
+
+    // handle last point which may not be part of a corner
+    {
+      const auto last_point = points.back();
+      const auto prev_point = *(points.cend() - 2);
+      const auto dx = last_point.x - prev_point.x;
+      const auto dy = last_point.y - prev_point.y;
+      const auto hypot = std::hypot(dx, dy);
+      const auto cosAlpha = dx / hypot;
+      const auto sinAlpha = dy / hypot;
+      // a polyline extends with half the stroke width in both perpendicular
+      // directions, but not along the line at its endpoints
+      const auto x_extension = std::abs(attributes.stroke_width / 2 * sinAlpha);
+      const auto y_extension = std::abs(attributes.stroke_width / 2 * cosAlpha);
+      const SVG::SVGRect last_point_bbox = {points.back().x - x_extension,
+                                            points.back().y - y_extension,
+                                            x_extension * 2, y_extension * 2};
+      m_bbox->extend(last_point_bbox);
+    }
 
     if (points.size() >= 3) {
       // at least one corner
@@ -842,7 +865,7 @@ SVG::SVGElement::miter_point(SVG::SVGPoint segment_start,
     const auto sinBetaMinusPi = -sinBeta;
     const auto cosBetaMinusPi = -cosBeta;
     const SVG::SVGPoint P2 = {P.x + stroke_width / 2.0 * sinBetaMinusPi,
-                              P.y + stroke_width / 2.0 * cosBetaMinusPi};
+                              P.y - stroke_width / 2.0 * cosBetaMinusPi};
 
     // the bevel is the triangle formed from the three points P, P1 and P2 so
     // a good enough approximation of the miter point in this case is the
